import ssl
import time
from datetime import datetime

from OpenSSL import crypto

from argos.checks.base import (
    BaseCheck,
    ExpectedIntValue,
    ExpectedStringValue,
    Response,
    Status,
    Severity,
)
from argos.logging import logger


class HTTPStatus(BaseCheck):
    config = "status-is"
    expected_cls = ExpectedIntValue

    async def run(self) -> dict:
        # XXX Get the method from the task
        task = self.task
        response = await self.http_client.request(method="get", url=task.url)

        return self.response(
            status=response.status_code == self.expected,
            expected=self.expected,
            retrieved=response.status_code,
        )


class HTTPBodyContains(BaseCheck):
    config = "body-contains"
    expected_cls = ExpectedStringValue

    async def run(self) -> dict:
        response = await self.http_client.request(method="get", url=self.task.url)
        return self.response(status=self.expected in response.text)


class SSLCertificateExpiration(BaseCheck):
    config = "ssl-certificate-expiration"
    expected_cls = ExpectedStringValue

    async def run(self):
        """Returns the number of days in which the certificate will expire."""
        response = await self.http_client.get(self.task.url)
        if response.is_error:
            raise

        network_stream = ssl_object = response.extensions["network_stream"]
        ssl_obj = network_stream.get_extra_info("ssl_object")
        cert = ssl_obj.getpeercert()

        not_after = datetime.strptime(cert.get("notAfter"), "%b %d %H:%M:%S %Y %Z")
        expires_in = (not_after - datetime.now()).days

        return self.response(status=Status.ON_CHECK, expires_in=expires_in)

    @classmethod
    async def finalize(cls, config, result, expires_in):
        thresholds = config.ssl.thresholds
        thresholds.sort()
        for days, severity in thresholds:
            if expires_in < days:
                return Status.FAILURE, severity
        return Status.SUCCESS, Severity.OK
