import os
from datetime import datetime
from enum import StrEnum
from typing import Dict, List, Literal, Optional, Tuple, Union

import yaml
from pydantic import BaseModel, Field, HttpUrl, validator
from yamlinclude import YamlIncludeConstructor

# XXX Find a way to check without having cirular imports

# This file contains the pydantic schemas. For the database models, check in argos.model.


Severity = Literal["warning", "error", "critical"]


class SSL(BaseModel):
    thresholds: List[Tuple[int, Severity]]

    @validator("thresholds", each_item=True, pre=True)
    def parse_threshold(cls, value):
        for duration_str, severity in value.items():
            num = int("".join(filter(str.isdigit, duration_str)))
            if "d" in duration_str:
                num = num
            elif "w" in duration_str:
                num = num * 7
            elif "m" in duration_str:
                num = num * 30
            else:
                raise ValueError("Invalid duration value")
            # Return here because it's one-item dicts.
            return (num, severity)


class WebsiteCheck(BaseModel):
    key: str
    value: str | List[str] | Dict[str, str]

    class Config:
        arbitrary_types_allowed = True

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, value):
        if isinstance(value, str):
            return {"expected": value}
        elif isinstance(value, dict):
            return value
        elif isinstance(value, list):
            return {"expected": value}
        else:
            raise ValueError("Invalid type")


class WebsitePath(BaseModel):
    path: str
    checks: List[Tuple[str, str | dict | int]]

    @validator("checks", each_item=True, pre=True)
    def parse_checks(cls, value):
        from argos.checks import \
            get_registered_checks  # To avoid circular imports

        available_names = get_registered_checks().keys()

        for name, expected in value.items():
            if name not in available_names:
                msg = f"Check should be one of f{available_names}. ({name} given)"
                raise ValueError(msg)
            return (name, expected)


class Website(BaseModel):
    domain: HttpUrl
    paths: List[WebsitePath]


class Service(BaseModel):
    port: int
    secrets: List[str]


class Alert(BaseModel):
    error: List[str]
    warning: List[str]
    alert: List[str]


class General(BaseModel):
    frequency: str
    alerts: Alert


class Config(BaseModel):
    general: General
    service: Service
    ssl: SSL
    websites: List[Website]


def validate_config(config: dict):
    return Config(**config)


def from_yaml(filename):
    parsed = load_yaml(filename)
    return validate_config(parsed)


def load_yaml(filename):
    base_dir = os.path.dirname(filename)
    YamlIncludeConstructor.add_to_loader_class(
        loader_class=yaml.FullLoader, base_dir=base_dir
    )

    with open(filename, "r") as stream:
        return yaml.load(stream, Loader=yaml.FullLoader)
