import sys
from typing import Annotated, List, Optional

from fastapi import Depends, FastAPI, Header, HTTPException, Request
from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer
from pydantic import BaseModel, ValidationError
from sqlalchemy.orm import Session

from argos.checks import get_registered_check
from argos.logging import logger
from argos.schemas import AgentResult, Task
from argos.schemas.config import from_yaml as get_schemas_from_yaml
from argos.server import models, queries
from argos.server.database import SessionLocal, engine
from argos.server.alerting import handle_alert

models.Base.metadata.create_all(bind=engine)

app = FastAPI()
auth_scheme = HTTPBearer()


def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


async def verify_token(token: HTTPAuthorizationCredentials = Depends(auth_scheme)):
    if token.credentials not in app.config.service.secrets:
        raise HTTPException(status_code=401, detail="Unauthorized")
    return token


@app.on_event("startup")
async def read_config_and_populate_db():
    # XXX Get filename from environment.
    try:
        config = get_schemas_from_yaml("config.yaml")
        app.config = config
    except ValidationError as e:
        logger.error(f"Errors where found while reading configuration:")
        for error in e.errors():
            logger.error(f"{error['loc']} is {error['type']}")
        sys.exit(1)

    db = SessionLocal()
    try:
        await queries.update_from_config(db, config)
    finally:
        db.close()


# XXX Get the default limit from the config
@app.get("/tasks", response_model=list[Task], dependencies=[Depends(verify_token)])
async def read_tasks(request: Request, db: Session = Depends(get_db), limit: int = 20):
    # XXX Let the agents specifify their names (and use hostnames)
    tasks = await queries.list_tasks(db, agent_id=request.client.host, limit=limit)
    return tasks


@app.post("/results", status_code=201, dependencies=[Depends(verify_token)])
async def create_result(results: List[AgentResult], db: Session = Depends(get_db)):
    """Get the results from the agents and store them locally.

    - Finalize the checks (some checks need the server to do some part of the validation,
      for instance because they need access to the configuration)
    - If it's an error, determine its severity ;
    - Trigger the reporting calls
    """
    db_results = []
    for agent_result in results:
        result = await queries.create_result(db, agent_result)
        # XXX Maybe offload this to a queue.
        # XXX Use a schema for the on-check value.
        # XXX Get all the tasks at once, to limit the queries on the db
        task = await queries.get_task(db, agent_result.task_id)
        if not task:
            logger.error(f"Unable to find task {agent_result.task_id}")
        else:
            check = task.get_check()
            status, severity = await check.finalize(
                app.config, result, **result.context
            )
            result.severity = severity
            result.status = status
            # Set the selection status to None
            task.selected_by = None
            handle_alert(app.config, result, task, severity)

        db_results.append(result)
    db.commit()
    return {"result_ids": [r.id for r in db_results]}


@app.get("/stats", dependencies=[Depends(verify_token)])
async def get_stats(db: Session = Depends(get_db)):
    return {
        "upcoming_tasks_count": await queries.count_tasks(db, selected=False),
        "results_count": await queries.count_results(db),
        "selected_tasks_count": await queries.count_tasks(db, selected=True),
    }
